==> qthello_0.7.tar.gz: qthello-0.7/src/centralwidget.cpp <==
#include <QApplication>
#include <QDesktopWidget>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QPair>
#include <QList>
#include <QtNetwork/QTcpServer>
#include <QtNetwork/QTcpSocket>
#include <QtNetwork/QHostAddress>
#include <QtNetwork/QNetworkInterface>

#include "centralwidget.h"
#include "fieldview.h"
#include "infopanel.h"
#include "modepanel.h"
#include "serverdialog.h"
#include "clientdialog.h"
#include "protocol.h"
#include "utils.h"

CentralWidget::CentralWidget(QWidget *parent, ReversiGame &reversiGame):
    QWidget(parent),
    gameServer_(0),
    gameClient_(0),
    gameConnection_(0),
    isNetworkGame_(false),
    reversiGame_(reversiGame)
{
    layout_ = new QGridLayout(this);
    fieldView_ = new FieldView(reversiGame_, this);
    layout_->addWidget(fieldView_, 0, 0);

    infoLayout_ = new QVBoxLayout();
    infoLayout_->addStretch(1);

    infoPanel_ = new InfoPanel(reversiGame_, this);
    infoLayout_->addWidget(infoPanel_);

    modePanel_ = new ModePanel(this);
    infoLayout_->addWidget(modePanel_);
    infoLayout_->addStretch(1);

    layout_->addItem(infoLayout_, 0, 1);

    showInfoPanels();

    connect(&reversiGame_, SIGNAL(stepUnavailable()), SLOT(stepUnavailable()));

    //network stuff
    gameServer_ = new QTcpServer(this);
    gameClient_ = new QTcpSocket(this);
    connect(gameClient_, SIGNAL(readyRead()), this, SLOT(slotReadNetworkData()));
    connect(gameClient_, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(displayP2PError(QAbstractSocket::SocketError)));
}//ctor

void CentralWidget::slotOrientationChanged()
{
    showInfoPanels();
}

void CentralWidget::showInfoPanels()
{
    QRect screenGeometry = QApplication::desktop()->screenGeometry();
    if (screenGeometry.width() > screenGeometry.height())
    {
        layout_->removeItem(infoLayout_);
        layout_->addItem(infoLayout_, 0, 1);
    }
    else
    {
        layout_->removeItem(infoLayout_);
        layout_->addItem(infoLayout_, 1, 0);
    }
}//showInfoPanels

void
CentralWidget::newGame()
{
    static int ai_levels[] = {2,5,6,8};
    QPair<int, int> mode = modePanel_->mode();
    gameMode_ = mode.first;
    strength_ = ai_levels[mode.second];
    if (mode.first == 3)
    {
        if (!newGameP2PServer())
            return;
    }
    else if (mode.first == 4)
    {
        if (!newGameP2PClient())
            return;
    }
    isNetworkGame_ = mode.first > 2;
    reversiGame_.init(gameMode_, strength_, isNetworkGame_);
}//newGame

bool CentralWidget::newGameP2PServer()
{
    if (!gameServer_->isListening())
        if (!gameServer_->listen())
        {
            showMessage(this, tr("Error!!\nUnable to start the game server: %1.").arg(gameServer_->errorString()), MSG_ERROR);
            return false;
        }

    ServerDialog serverDialog(gameServer_, &gameConnection_, isNetworkGame_, this);
    int res = serverDialog.exec();
    if (res == QDialog::Rejected)
        return false;
    connect(gameConnection_, SIGNAL(readyRead()), this, SLOT(slotReadNetworkData()));
    connect(gameConnection_, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(displayP2PError(QAbstractSocket::SocketError)));
    writeP2PData(gameConnection_, QString(PROTOCOL_VERSION));
    //TODO: wait answer from client
    P2PState_ = WAIT_CLIENT_VERSION;
    return true;
}

bool CentralWidget::newGameP2PClient()
{
    ClientDialog clientDialog(gameClient_, this);
    int res = clientDialog.exec();
    if (res == QDialog::Rejected)
        return false;
    P2PState_ = WAIT_SERVER_VERSION;
    return true;
}

void
CentralWidget::slotReadNetworkData()
{
    QString data;
    if (gameMode_ == 4)
        data = readP2PData(gameClient_);
    else
        data = readP2PData(gameConnection_);
    if (!data.isEmpty())
    {
        //qDebug("st %d", P2PState_);
        //qDebug() << data;
        if (P2PState_ == WAIT_CLIENT_VERSION && data.at(0) == 'v')
        {
            writeP2PData(gameConnection_, QString("newgame"));
            P2PState_ = RUNNING;
            reversiGame_.init(gameMode_, strength_, isNetworkGame_);
        }
        else if (P2PState_ == WAIT_SERVER_VERSION && data.at(0) == 'v')
        {
            writeP2PData(gameClient_, QString(PROTOCOL_VERSION));
            P2PState_ = WAIT_SERVER_NEW_GAME;
        }
        else if (P2PState_ == WAIT_SERVER_NEW_GAME && data.at(0) == 'n')
        {
            P2PState_ = RUNNING;
            reversiGame_.init(gameMode_, strength_, isNetworkGame_);
        }
        else if ((P2PState_ == CLIENT_PAUSED || P2PState_ == SERVER_PAUSED) && data.at(0) == 'u')
        {
            P2PState_ = RUNNING;
        }
        else if (data.at(0) == 'x')
        {
            QChar x, y;
            x = data.at(1);
            y = data.at(3);
            reversiGame_.makeStep(x.toAscii() - '0', y.toAscii() - '0');
        }
    }
}//slotReadNetworkData

void
CentralWidget::slotPlayerStep(int x, int y)
{
    if (P2PState_ != RUNNING)
    {
        switch (P2PState_)
        {
            case WAIT_CLIENT_VERSION:
                showMessage(this, tr("Warning!\nWait answer from client."), MSG_WARNING);
                break;
            case CLIENT_PAUSED:
                showMessage(this, tr("Warning!\nClient is paused."), MSG_WARNING);
                break;
            case SERVER_PAUSED:
                showMessage(this, tr("Warning!\nServer is paused."), MSG_WARNING);
                break;
            case WAIT_SERVER_VERSION:
            case WAIT_SERVER_NEW_GAME:
                showMessage(this, tr("Warning!\nWait answer from server."), MSG_WARNING);
                break;
        }
        return;
    }//(P2PState_ != RUNNING)

    if (gameMode_ == 3 && reversiGame_.currentPlayer() == White)
    {
        showMessage(this, tr("Warning!\nIt's White player turn now."), MSG_WARNING);
        return;
    }
    if (gameMode_ == 4 && reversiGame_.currentPlayer() == Black)
    {
        showMessage(this, tr("Warning!\nIt's Black player turn now."), MSG_WARNING);
        return;
    }

    QString data("x%1y%2");
    reversiGame_.makeStep(x, y);
    if (gameMode_ == 3)
        writeP2PData(gameConnection_, data.arg(x).arg(y));
    else if (gameMode_ == 4)
        writeP2PData(gameClient_, data.arg(x).arg(y));
}//slotPlayerStep

void CentralWidget::stepUnavailable()
{
    static const char *players[] = {QT_TR_NOOP("White"), QT_TR_NOOP("Black")};
    QString info;
    info = QString(tr("%1 can't move. %2 turn.")).arg(tr(players[reversiGame_.currentPlayer()])).arg(tr(players[reversiGame_.opponentPlayer()]));

    //set pause while message will be shown
    if (isNetworkGame_)
    {
        if (gameMode_ == 4)
        {
            P2PState_ = SERVER_PAUSED;
        }
        else if (gameMode_ == 3)
        {
            P2PState_ = CLIENT_PAUSED;
        }
    }//if (isNetworkGame_)

    showMessage(parentWidget(), info, MSG_WARNING);

    //set perr about unpaused
    if (isNetworkGame_)
    {
        if (gameMode_ == 4)
        {
            writeP2PData(gameClient_, QString("unpaused"));
        }
        else if (gameMode_ == 3)
        {
            writeP2PData(gameConnection_, QString("unpaused"));
        }
    }//if (isNetworkGame_)
}//stepUnavailable

void CentralWidget::displayP2PError(QAbstractSocket::SocketError socketError)
{
    switch (socketError) {
        case QAbstractSocket::HostNotFoundError:
            showMessage(parentWidget(), tr("Warning!\nThe host was not found."), MSG_WARNING);
            break;
        case QAbstractSocket::ConnectionRefusedError:
            showMessage(parentWidget(), tr("Warning!\nThe connection was refused by the peer."), MSG_WARNING);
            break;
        default:
            QString errmsg(tr("The following error occurred: %1."));
            if (gameMode_ == 3)
                showMessage(parentWidget(), errmsg.arg(gameConnection_->errorString()));
            else
                showMessage(parentWidget(), errmsg.arg(gameClient_->errorString()));
    }
}//displayP2PError

==> qthello_0.7.2.tar.gz: qthello-0.7.2/src/centralwidget.cpp <==
#include <QApplication>
#include <QDesktopWidget>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QPair>
#include <QList>
#include <QtNetwork/QTcpServer>
#include <QtNetwork/QTcpSocket>
#include <QtNetwork/QHostAddress>
#include <QtNetwork/QNetworkInterface>

#include "centralwidget.h"
#include "fieldview.h"
#include "infopanel.h"
#include "modepanel.h"
#include "serverdialog.h"
#include "clientdialog.h"
#include "protocol.h"
#include "utils.h"
#include "reversidefs.h"

CentralWidget::CentralWidget(QWidget *parent, ReversiGame &reversiGame):
    QWidget(parent),
    gameServer_(0),
    gameClient_(0),
    gameConnection_(0),
    isNetworkGame_(false),
    reversiGame_(reversiGame)
{
    layout_ = new QGridLayout(this);
    fieldView_ = new FieldView(reversiGame_, this);
    layout_->addWidget(fieldView_, 0, 0);

    infoLayout_ = new QVBoxLayout();
    infoLayout_->addStretch(1);
    infoPanel_ = new InfoPanel(reversiGame_, this);
    infoLayout_->addWidget(infoPanel_);

    modePanel_ = new ModePanel(this);
    infoLayout_->addWidget(modePanel_);
    infoLayout_->addStretch(1);

    layout_->addItem(infoLayout_, 0, 1);

    showInfoPanels();

    connect(&reversiGame_, SIGNAL(stepUnavailable()), SLOT(stepUnavailable()));

    //network stuff
    gameServer_ = new QTcpServer(this);
    gameClient_ = new QTcpSocket(this);
    connect(gameClient_, SIGNAL(readyRead()), this, SLOT(slotReadNetworkData()));
    connect(gameClient_, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(displayP2PError(QAbstractSocket::SocketError)));
}//ctor

void CentralWidget::slotOrientationChanged()
{
    showInfoPanels();
}

void CentralWidget::showInfoPanels()
{
    QRect screenGeometry = QApplication::desktop()->screenGeometry();
    if (screenGeometry.width() > screenGeometry.height())
    {
        layout_->removeItem(infoLayout_);
        layout_->addItem(infoLayout_, 0, 1);
    }
    else
    {
        layout_->removeItem(infoLayout_);
        layout_->addItem(infoLayout_, 1, 0);
    }
}//showInfoPanels

void
CentralWidget::newGame()
{
    static int ai_levels[] = {2,5,6,MAX_AI_LEVEL};
    QPair<int, int> mode = modePanel_->mode();
    gameMode_ = mode.first;
    strength_ = ai_levels[mode.second];
    if (mode.first == 3)
    {
        if (!newGameP2PServer())
            return;
    }
    else if (mode.first == 4)
    {
        if (!newGameP2PClient())
            return;
    }
    isNetworkGame_ = mode.first > 2;
    reversiGame_.init(gameMode_, strength_, isNetworkGame_);
}//newGame

bool CentralWidget::newGameP2PServer()
{
    if (!gameServer_->isListening())
        if (!gameServer_->listen())
        {
            showMessage(this, tr("Error!!\nUnable to start the game server: %1.").arg(gameServer_->errorString()), MSG_ERROR);
            return false;
        }

    ServerDialog serverDialog(gameServer_, &gameConnection_, isNetworkGame_, this);
    int res = serverDialog.exec();
    if (res == QDialog::Rejected)
        return false;
    connect(gameConnection_, SIGNAL(readyRead()), this, SLOT(slotReadNetworkData()));
    connect(gameConnection_, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(displayP2PError(QAbstractSocket::SocketError)));
    writeP2PData(gameConnection_, QString(PROTOCOL_VERSION));
    //TODO: wait answer from client
    P2PState_ = WAIT_CLIENT_VERSION;
    return true;
}

bool CentralWidget::newGameP2PClient()
{
    ClientDialog clientDialog(gameClient_, this);
    int res = clientDialog.exec();
    if (res == QDialog::Rejected)
        return false;
    P2PState_ = WAIT_SERVER_VERSION;
    return true;
}

void
CentralWidget::slotReadNetworkData()
{
    QString data;
    if (gameMode_ == 4)
        data = readP2PData(gameClient_);
    else
        data = readP2PData(gameConnection_);
    if (!data.isEmpty())
    {
        //qDebug("st %d", P2PState_);
        //qDebug() << data;
        if (P2PState_ == WAIT_CLIENT_VERSION && data.at(0) == 'v')
        {
            writeP2PData(gameConnection_, QString("newgame"));
            P2PState_ = RUNNING;
            reversiGame_.init(gameMode_, strength_, isNetworkGame_);
        }
        else if (P2PState_ == WAIT_SERVER_VERSION && data.at(0) == 'v')
        {
            writeP2PData(gameClient_, QString(PROTOCOL_VERSION));
            P2PState_ = WAIT_SERVER_NEW_GAME;
        }
        else if (P2PState_ == WAIT_SERVER_NEW_GAME && data.at(0) == 'n')
        {
            P2PState_ = RUNNING;
            reversiGame_.init(gameMode_, strength_, isNetworkGame_);
        }
        else if ((P2PState_ == CLIENT_PAUSED || P2PState_ == SERVER_PAUSED) && data.at(0) == 'u')
        {
            P2PState_ = RUNNING;
        }
        else if (data.at(0) == 'x')
        {
            QChar x, y;
            x = data.at(1);
            y = data.at(3);
            reversiGame_.makeStep(x.toAscii() - '0', y.toAscii() - '0');
        }
    }
}//slotReadNetworkData

void
CentralWidget::slotPlayerStep(int x, int y)
{
    if (P2PState_ != RUNNING)
    {
        switch (P2PState_)
        {
            case WAIT_CLIENT_VERSION:
                showMessage(this, tr("Warning!\nWait answer from client."), MSG_WARNING);
                break;
            case CLIENT_PAUSED:
                showMessage(this, tr("Warning!\nClient is paused."), MSG_WARNING);
                break;
            case SERVER_PAUSED:
                showMessage(this, tr("Warning!\nServer is paused."), MSG_WARNING);
                break;
            case WAIT_SERVER_VERSION:
            case WAIT_SERVER_NEW_GAME:
                showMessage(this, tr("Warning!\nWait answer from server."), MSG_WARNING);
                break;
        }
        return;
    }//(P2PState_ != RUNNING)

    if (gameMode_ == 3 && reversiGame_.currentPlayer() == White)
    {
        showMessage(this, tr("Warning!\nIt's White player turn now."), MSG_WARNING);
        return;
    }
    if (gameMode_ == 4 && reversiGame_.currentPlayer() == Black)
    {
        showMessage(this, tr("Warning!\nIt's Black player turn now."), MSG_WARNING);
        return;
    }

    QString data("x%1y%2");
    reversiGame_.makeStep(x, y);
    if (gameMode_ == 3)
        writeP2PData(gameConnection_, data.arg(x).arg(y));
    else if (gameMode_ == 4)
        writeP2PData(gameClient_, data.arg(x).arg(y));
}//slotPlayerStep

void CentralWidget::stepUnavailable()
{
    static const char *players[] = {QT_TR_NOOP("White"), QT_TR_NOOP("Black")};
    QString info;
    info = QString(tr("%1 can't move. %2 turn.")).arg(tr(players[reversiGame_.currentPlayer()])).arg(tr(players[reversiGame_.opponentPlayer()]));

    //set pause while message will be shown
    if (isNetworkGame_)
    {
        if (gameMode_ == 4)
        {
            P2PState_ = SERVER_PAUSED;
        }
        else if (gameMode_ == 3)
        {
            P2PState_ = CLIENT_PAUSED;
        }
    }//if (isNetworkGame_)

    showMessage(parentWidget(), info, MSG_WARNING);

    //set perr about unpaused
    if (isNetworkGame_)
    {
        if (gameMode_ == 4)
        {
            writeP2PData(gameClient_, QString("unpaused"));
        }
        else if (gameMode_ == 3)
        {
            writeP2PData(gameConnection_, QString("unpaused"));
        }
    }//if (isNetworkGame_)
}//stepUnavailable

void CentralWidget::displayP2PError(QAbstractSocket::SocketError socketError)
{
    switch (socketError) {
        case QAbstractSocket::HostNotFoundError:
            showMessage(parentWidget(), tr("Warning!\nThe host was not found."), MSG_WARNING);
            break;
        case QAbstractSocket::ConnectionRefusedError:
            showMessage(parentWidget(), tr("Warning!\nThe connection was refused by the peer."), MSG_WARNING);
            break;
        default:
            QString errmsg(tr("The following error occurred: %1."));
            if (gameMode_ == 3)
                showMessage(parentWidget(), errmsg.arg(gameConnection_->errorString()));
            else
                showMessage(parentWidget(), errmsg.arg(gameClient_->errorString()));
    }
}//displayP2PError
